/************************************************************************/
/* der.h - DER encoding/decoding Header					*/
/************************************************************************/
/*
 *  (c)2007 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _DER_H_
#define _DER_H_

#include <cutil/cutil_portable.h>


#define DER_TYPE_BOOLEAN		0x01
#define DER_TYPE_INTEGER		0x02
#define DER_TYPE_BITSTRING		0x03
#define DER_TYPE_OCTETSTRING		0x04
#define DER_TYPE_NULL			0x05
#define DER_TYPE_OID			0x06
#define DER_TYPE_SEQUENCE		0x10
#define DER_TYPE_SET			0x11

#define DER_CLASS_UNIVERSAL		0x00
#define DER_CLASS_APPLICATION		0x40
#define DER_CLASS_CONTEXT		0x80
#define DER_CLASS_PRIVATE		0xc0

#define DER_MASK_CLASS			0xc0
#define DER_MASK_FORMAT			0x20
#define DER_MASK_DATATYPE		0x1f

#define DER_GET_IDENT_CLASS(x)		(x & DER_MASK_CLASS)
#define DER_GET_IDENT_FORMAT(x)		(x & DER_MASK_FORMAT)
#define DER_GET_IDENT_DATATYPE(x)	(x & DER_MASK_DATATYPE)

#define DER_IS_PRIMITIVE(x)		( x & DER_MASK_FORMAT ? 0 : 1 )
#define DER_IS_CONSTRUCTED(x)		( x & DER_MASK_FORMAT ? 1 : 0 )

typedef struct _DERIdentifier	DERIdentifier;
struct _DERIdentifier {
	DERIdentifier		*previous;
	DERIdentifier		*next;
	DERIdentifier		*child;
	unsigned char		identifier;
	unsigned int		length;
	union {
		long		l;
		long long	ll;
		char		*s;
	};
};

CUTIL_API int der_identifier_new (DERIdentifier **dip);
CUTIL_API int der_identifier_new_boolean (DERIdentifier **dip, int value);
CUTIL_API int der_identifier_new_integer (DERIdentifier **dip, int value);
CUTIL_API int der_identifier_new_string (DERIdentifier **dip, char *string, int length);
CUTIL_API int der_identifier_new_null (DERIdentifier **dip);
CUTIL_API int der_identifier_new_oid (DERIdentifier **dip, char *string, int length);
CUTIL_API int der_identifier_new_sequence (DERIdentifier **dip);
CUTIL_API int der_identifier_free (DERIdentifier **dip);

CUTIL_API int der_identifier_get_boolean (DERIdentifier *di, int *valuep);
CUTIL_API int der_identifier_get_integer (DERIdentifier *di, int *valuep);
CUTIL_API int der_identifier_get_octetstring (DERIdentifier *di, char *str, int *lengthp);
CUTIL_API int der_identifier_get_oid (DERIdentifier *di, char *str, int *lengthp);

CUTIL_API int der_identifier_chain_append (DERIdentifier *parent, DERIdentifier *di);
CUTIL_API int der_identifier_chain_prepend (DERIdentifier *parent, DERIdentifier *di);
CUTIL_API int der_identifier_chain_free (DERIdentifier **dip);

CUTIL_API int der_identifier_child_append (DERIdentifier *parent, DERIdentifier *di);
CUTIL_API int der_identifier_child_prepend (DERIdentifier *parent, DERIdentifier *di);

CUTIL_API int der_identifier_encode (DERIdentifier *di, char **datap, int *datalenp);
CUTIL_API int der_identifier_decode (DERIdentifier **dip, char *data, int datalen);

#endif


