/************************************************************************/
/* xmlrpc_p.h - XMLRPC Implementation Internal Header			*/
/************************************************************************/
/*
 *  (c)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _XMLRPC_P_H_
#define _XMLRPC_P_H_


#include "slist.h"

/* Keep in sync with the defines in xmlrpc.h */
#define XMLRPC_VALUETYPE_INT		0x01
#define XMLRPC_VALUETYPE_BOOL		0x02
#define XMLRPC_VALUETYPE_STRING		0x03
#define XMLRPC_VALUETYPE_DOUBLE		0x04
#define XMLRPC_VALUETYPE_DATETIME	0x05
#define XMLRPC_VALUETYPE_BASE64		0x06
#define XMLRPC_VALUETYPE_STRUCT		0x07
#define XMLRPC_VALUETYPE_ARRAY		0x08
/* Keep in sync with the defines in xmlrpc.h */

#define XMLRPC_VALUETYPE_MIN		0x01
#define XMLRPC_VALUETYPE_MAX		0x08

#define XMLRPC_RESPONSETYPE_NORMAL	0x01
#define XMLRPC_RESPONSETYPE_FAULT	0x02

#define XMLRPC_STRBUFFER_SIZE		256

/* XMLRPC Primitives */
typedef struct _XmlrpcRef XmlrpcRef;
struct _XmlrpcRef {
	unsigned int refcount; /* When reduced to zero, free memory */
};


typedef struct _XmlrpcValue XmlrpcValue;
struct _XmlrpcValue {
	XmlrpcRef ref; /* for reference counting */
	int type;    /* one of XMLRPC_VALUETYPE_* */
	int datalen; /* int/bool/struct/array=4, string=x, base64=x, etc */
	void *data;  /* always a pointer to allocated memory!!! */
};

typedef struct _XmlrpcStructMember XmlrpcStructMember;
struct _XmlrpcStructMember {
	XmlrpcValue *value;
	char *name;
};

typedef struct _XmlrpcStruct XmlrpcStruct;
struct _XmlrpcStruct {
	XmlrpcRef ref; /* for reference counting */
	SList *memberlist;	/* single-linked list of XmlrpcStructMember structs */
};


typedef struct _XmlrpcArray XmlrpcArray;
struct _XmlrpcArray {
	XmlrpcRef ref; /* for reference counting */
	SList *valuelist;	/* single-linked list of XmlrpcValue structs */
};


typedef struct _XmlrpcRequest XmlrpcRequest;
struct _XmlrpcRequest {
	char *methodname;
	SList *paramlist;	/* single-linked list of XmlrpcValue structs */
};

typedef struct _XmlrpcResponse XmlrpcResponse;
struct _XmlrpcResponse {
	int type;	  /* Response or Fault */
	char *methodname; /* the method we're responsing to (for future use) */
	SList *paramlist; /* single-linked list of XmlrpcValue structs */
			  /* A response SHOULD only contain a single <value>, */
			  /* but maybe a future version will allow multiples. */
};


#endif
