/************************************************************************/
/* attribute.c - Attribute API Source					*/
/************************************************************************/
/*
 * (C)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/
/* History:								*/
/************************************************************************/


#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <cutil/attribute.h>
#include <cutil/cutil_portable.h>

#ifdef	DMALLOC
#include <dmalloc.h>
#endif



/************************************************************************/
/* AList API Function Declarations					*/
/************************************************************************/
CUTIL_API Attribute *attribute_add(Attribute *attrib, char *name, char *value)
{
	Attribute *a = NULL;

	if (name == NULL)
		return NULL;

	if (attrib != NULL)
		if (attribute_find(attrib, name))
			return attrib;

	a = calloc(1, sizeof(Attribute));
	if (a == NULL) {
		if (attrib != NULL)
			return attrib;
		else
			return NULL;
	}
	a->name = strdup(name);
	if (value != NULL)
		a->value = strdup(value);
	if (attrib != NULL)
		a->next = attrib;
	return a;
}


CUTIL_API Attribute *attribute_remove(Attribute *attrib, char *name)
{
	Attribute *a = NULL;
	Attribute *pa = NULL;

	if (attrib == NULL)
		return NULL;
	if (name == NULL)
		return attrib;
	a = attrib;
	while (a != NULL) {
		if(!strcmp(a->name, name)) {
			if(a->name != NULL)
				free(a->name);
			if(a->value != NULL)
				free(a->value);
			if (pa != NULL) { /* not first in list */
				pa->next = a->next;
				free(a);
				return attrib;
			} else {	/* first in list */
				pa = a->next;
				free(a);
				return pa;
			}
		}
		pa = a;
		a = a->next;
	}
	return a;
}



CUTIL_API Attribute *attribute_find(Attribute *attrib, char *name)
{
	Attribute *a = NULL;
	if (attrib == NULL || name == NULL)
		return NULL;
	a = attrib;
	while (a != NULL) {
		if(!strcmp(a->name, name))
			return a;
		a = a->next;
	}
	return NULL;
}


CUTIL_API char *attribute_get_value(Attribute *attrib, char *name)
{
	Attribute *a = NULL;
	if (attrib == NULL || name == NULL)
		return NULL;
	a = attribute_find(attrib, name);
	if (a == NULL)
		return NULL;
	return a->value;
}


CUTIL_API int attribute_get_name_and_value (Attribute *attrib, char **name,
				char **value)
{
	if (attrib == NULL || name == NULL || value == NULL)
		return EINVAL;
	*name = attrib->name;
	*value = attrib->value;
	return 0;
}




CUTIL_API int	attribute_foreach(Attribute *attrib,
				AttributeCallbackFunction callback,
				void *data)
{
	int rc;
	int count = 0;
	Attribute *a = NULL;
	if (attrib == NULL || callback == NULL)
		return 0;
	a = attrib;
	while (a != NULL) {
		rc = (*callback)(a, data);
		if (rc != 0)
			return rc;
		a = a->next;
		count++;
	}
	return 0;
}


CUTIL_API void	attribute_freeall(Attribute **attrib)
{
	Attribute *a;
	Attribute *na;
	if (attrib == NULL || *attrib == NULL)
		return;
	a = *attrib;
	while (a != NULL) {
		na = a->next;
		if (a->name != NULL)
			free(a->name);
		if (a->value != NULL)
			free(a->value);
		free(a);
		a = na;
	}
	*attrib = NULL;
	return;
}


