/************************************************************************/
/* base64.c - base64 encode/decode functions source			*/
/************************************************************************/
/*
 *  (c)2002 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>

#include <cutil/base64.h>

#ifdef	DMALLOC
#include <dmalloc.h>
#endif


#define CHARS_PER_LINE		72

static const char b64char[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
			"abcdefghijklmnopqrstuvwxyz"
			"0123456789+/";




/*GString *base64_encode_buffer(char *buffer, int length) */
CUTIL_API int base64_encode(char *indata, int inlen, char **outdata, int *outlen)
{

	int div;
	int rem;
	int chars;
	int newlines;
	int obytes;
	char *buf = NULL;
	char *outbuffer = NULL;
	const char *data = NULL;

	if (indata == NULL || outdata == NULL || outlen == NULL)
		return EINVAL;
	*outdata = NULL;
	*outlen = 0;

	obytes = 0;
	div = inlen / 3;
	rem = inlen % 3;
	chars = div * 4 + rem + 1;
	newlines = (chars + CHARS_PER_LINE - 1) / CHARS_PER_LINE;

	data = indata;

	outbuffer = (char *)calloc(1, chars + newlines + 4);
	if (outbuffer == NULL)
		return ENOMEM;

	chars = 0;
	buf = outbuffer;
	while(div > 0) {
		buf[0] = b64char[(data[0] >> 2) & 0x3f];
		buf[1] = b64char[((data[0] << 4) & 0x30) +
			((data[1] >> 4) & 0xf)];
		buf[2] = b64char[((data[1] << 2) & 0x3c) +
			((data[2] >> 6) & 0x3)];
		buf[3] = b64char[data[2] & 0x3f];
		data += 3;
		buf += 4;
		div--;
		chars += 4;
		if(chars == CHARS_PER_LINE) {
			*(buf++) = '\n';
			obytes += (chars + 1);
			chars = 0;
		}
	}
	switch(rem) {
		case 2:
			buf[0] = b64char[(data[0] >> 2) & 0x3f];
			buf[1] = b64char[((data[0] << 4) & 0x30) +
				((data[1] >> 4) & 0xf)];
			buf[2] = b64char[(data[1] << 2) & 0x3c];
			buf[3] = '=';
			buf += 4;
			chars +=4;
			break;
		case 1:
			buf[0] = b64char[(data[0] >> 2) & 0x3f];
			buf[1] = b64char[(data[0] << 4) & 0x30];
			buf[2] = '=';
			buf[3] = '=';
			buf += 4;
			chars +=4;
			break;
	}
	*buf = '\0';
	obytes += chars;
	*outdata = outbuffer;
	*outlen = obytes;
	return 0;
}


CUTIL_API int base64_decode(char *indata, int inlen, char **outdata, int *outlen)
{
	int len;
	int debyte;
	char bits;
	int gbyte;
	int obytes;
	char lastbyte;
	char *ostr =  NULL;
	char *str = NULL;

	if (indata == NULL || outdata == NULL || outlen == NULL)
		return EINVAL;
	*outdata = NULL;
	*outlen = 0;

	len = (inlen / 4) * 3;
	str = (char *)calloc(1, len+1);
	if (str == NULL)
		return ENOMEM;

	ostr = str;
	bits = 0;
	gbyte = 0;	
	obytes = 0;	
	lastbyte = 0;
	for(debyte = 0;debyte < inlen;debyte++) {
		if(indata[debyte] >= 'A' && indata[debyte] <= 'Z')
			bits = indata[debyte] - 'A';
		else if(indata[debyte] >= 'a' && indata[debyte] <= 'z')
			bits = indata[debyte] - 'a' + 26;
		else if(indata[debyte] >= '0' && indata[debyte] <= '9')
			bits = indata[debyte] - '0' + 52;
		else if(indata[debyte] == '+')
			bits = 62;
		else if(indata[debyte] == '/')
			bits = 63;
		else if(indata[debyte] == '=') {
			if( (inlen - debyte) == 1)
				obytes += 2;
			else if( (inlen - debyte) == 2)
				obytes++;
			break;
		}
		else if(indata[debyte] == '\n')
			continue;
		switch(gbyte) {
			case 0:
				ostr[0] = (bits << 2) & 0xfc;
				break;
			case 1:
				ostr[0] |= (bits >> 4) & 0x03;
				ostr[1] = (bits << 4) & 0xf0;
				break;
			case 2:
				ostr[1] |= (bits >> 2) & 0x0f;
				ostr[2] = (bits << 6) & 0xc0;
				break;
			case 3:
				ostr[2] |= bits & 0x3f;
		}
		gbyte++;
		if(gbyte == 4) {
			gbyte = 0;
			ostr += 3;
			obytes += 3;
		}
		lastbyte = indata[debyte];
	}
	*outdata = str;
	*outlen = obytes;
	return 0;
}


