/****************************************************************/
/* slist.c - Single-Linked List Source				*/
/****************************************************************/
/*
 *  (c)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/****************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <cutil/slist.h>

#ifdef	DMALLOC
#include <dmalloc.h>
#endif


CUTIL_API int slist_free (SList **listp)
{
	SList *l = NULL;
	SList *l2 = NULL;

	if (listp == NULL)
		return EINVAL;

	l = *listp;
	while (l != NULL) {
		l2 = l->next;
		free (l);
		l = l2;
	}
	*listp = NULL;
	return ESUCCESS;
}


CUTIL_API int slist_item_add (SList **listp, void *data)
{
	SList *l = NULL;
	SList *ln = NULL;

	if (listp == NULL || data == NULL)
		return EINVAL;

	ln = (SList *)calloc (1, sizeof (SList));
	if (ln == NULL)
		return ENOMEM;
	ln->data = data;

	if (*listp != NULL) {
		l = *listp;
		while (l->next != NULL)
			l = l->next;
		l->next = ln;
	} else {
		*listp = ln;
	}
	
	return ESUCCESS;
}


CUTIL_API int slist_item_get (SList *list, int index, void **data)
{
	int count;
	SList *l = NULL;

	if (list == NULL || data == NULL)
		return EINVAL;

	count = 0;
	l = list;
	while (l != NULL) {
		if (count == index) {
			*data = l->data;
			return ESUCCESS;
		}
		count++;
		l = l->next;
	}

	return ENOENT;
}


CUTIL_API int slist_item_count (SList *list, int *count)
{
	int c;
	SList *l = NULL;

	if (list == NULL || count == NULL)
		return EINVAL;

	c = 0;
	l = list;
	while (l != NULL) {
		
		c++;
		l = l->next;
	}
	*count = c;
	return ESUCCESS;
}


CUTIL_API int slist_item_remove (SList **listp, void *data)
{
	SList *l = NULL;
	SList *l1 = NULL;

	if (listp == NULL || *listp == NULL || data == NULL)
		return EINVAL;

	l = *listp;
	while (l != NULL) {
		if (l->data == data) {
			if (l1 == NULL)
				*listp = l->next;
			else
				l1->next = l->next;
			free(l);
			return ESUCCESS;
		}
		l1 = l;
		l = l->next;
	}

	return ENOENT;
}


CUTIL_API SList *slist_get_next (SList *list)
{
	if (list == NULL)
		return NULL;
	return list->next;
}


CUTIL_API void *slist_get_data (SList *list)
{
	if (list == NULL)
		return NULL;
	return list->data;
}

