/****************************************************************/
/* squeue.c - Static Queue Source				*/
/****************************************************************/
/*
 *  (c)2005 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/****************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <cutil/squeue.h>

#ifdef	DMALLOC
#include <dmalloc.h>
#endif


CUTIL_API int squeue_new (SQueue **queuep,
		unsigned long itemsize,
		unsigned long maxitems)
{
	SQueue	*queue = NULL;
	if (queuep == NULL || itemsize == 0 || maxitems == 0)
		return EINVAL;
	*queuep = NULL;
	queue = calloc (1, sizeof (SQueue));
	if ( queue == NULL)
		return ENOMEM;
	queue->queuedata = calloc (1, itemsize * maxitems);
	if (queue->queuedata == NULL) {
		free (queue);
		return ENOMEM;
	}
	queue->itemsize = itemsize;
	queue->queuesize = maxitems;
	*queuep = queue;
	return ESUCCESS;
}


CUTIL_API int squeue_free (SQueue **queuep)
{
	SQueue	*queue = NULL;
	if (queuep == NULL || *queuep == NULL)
		return EINVAL;
	queue = *queuep;
	if (queue->queuedata != NULL)
		free (queue->queuedata);
	free (queue);
	*queuep = NULL;
	return ESUCCESS;
}


CUTIL_API int squeue_item_enqueue (SQueue *queue, void *data)
{
	char 	*p = NULL;
	if (queue == NULL || data == NULL)
		return EINVAL;
	if (queue->items == queue->queuesize)
		return ENOSPC;
	p = queue->queuedata + queue->writepos;
	memcpy (p, data, queue->itemsize);
	queue->writepos += queue->itemsize;
	if (queue->writepos >= (queue->itemsize * queue->queuesize))
		queue->writepos = 0;
	queue->items++;
	return ESUCCESS;
}


CUTIL_API int squeue_item_dequeue (SQueue *queue, void *data)
{
	char 	*p = NULL;
	if (queue == NULL || data == NULL)
		return EINVAL;
	if (queue->items == 0)
		return ENOENT;
	p = queue->queuedata + queue->readpos;
	memcpy (data, p, queue->itemsize);
	queue->readpos += queue->itemsize;
	if (queue->readpos >= (queue->itemsize * queue->queuesize))
		queue->readpos = 0;
	queue->items--;
	return ESUCCESS;
}


CUTIL_API int squeue_item_peek (SQueue *queue, void *data)
{
	char 	*p = NULL;
	if (queue == NULL || data == NULL)
		return EINVAL;
	if (queue->readpos == queue->writepos)
		return ENOENT;
	p = queue->queuedata + queue->readpos;
	memcpy (data, p, queue->itemsize);
	return ESUCCESS;
}


CUTIL_API int squeue_item_count (SQueue *queue, int *countp)
{
	if (queue == NULL || countp == NULL)
		return EINVAL;
	*countp = queue->items;
	return ESUCCESS;
}


