/************************************************************************/
/* strfuncs.c - String manipulation functions source			*/
/************************************************************************/
/*
 *  (c)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#ifdef	DMALLOC
#include <dmalloc.h>
#endif

#include <cutil/strfuncs.h>


CUTIL_API int strnsplit (const char *str, int strlen, char delimiter,
		char ***destarray, int *destitems)
{
	char **array = NULL;
	char *nstr = NULL;
	char *p = NULL;
	char *pp = NULL;
	char *sp = NULL;
	int items;
	int offset;
	int len;

	if (str == NULL || strlen < 1 ||
	    destarray == NULL || destitems == NULL ||
	    *destitems < 1)
		return EINVAL;

	*destarray = NULL;
	array = (char **)calloc ((*destitems) + 1, sizeof(char *));
	if (array == NULL)
		return ENOMEM;

	nstr = (char *)calloc (1, strlen+1);
	if (nstr == NULL) {
		free (array);
		return ENOMEM;
	}
	memcpy (nstr, str, strlen);

	items = 0;
	offset = 0;
	p = nstr;
	sp = p;
	pp = p;
	while (offset <= strlen && items < (*destitems)) {
		if (*p == delimiter || offset == strlen) {
			if(*pp != delimiter) {
				len = (p - sp);
				if (items == (*destitems)-1)
					len += strlen - (p - nstr);
				array[items] = (char *)calloc (1, len+1);
				if (array[items] == NULL) {
					free (nstr);
					*destitems = items;
					return ENOMEM;
				}
				memcpy (array[items], sp, len);
				items++;
			}
		} else {
			if (*pp == delimiter) {
				sp = p;
			}
		}
		pp = p;
		p++;
		offset++;
	}

	*destarray = array;
	*destitems = items;
	free (nstr);
	return 0;
}


CUTIL_API int strarray_free (char ***strarray)
{
	char **array = NULL;
	int item = 0;;
	if (strarray == NULL || *strarray == NULL)
		return EINVAL;
	array = *strarray;
	while (array[item] != NULL) {
		free (array[item]);
		item++;
	}
	free (array);
	*strarray = NULL;
	return 0;
}


CUTIL_API int strnchug (char *str, int len)
{
	int i;
	char *p = NULL;
	char *c = NULL;
	if (str == NULL || len < 1)
		return EINVAL;
	p = str;
	c = str;
	i = 0;
	/* find the first non-whitespace character */
	while ( (*c != 0) && i < len) {
		if ( *c != ' ')
			break;
		c++;
		i++;
	}
	/* no leading spaces to remove */
	if (c == str)
		return 0;
	/* Did we reach the end of the buffer without finding a char? */
	if ( (*c == 0) || i == len) {
		*p = 0;
		return 0;
	}
	/* move the chars down to the start of the buffer */
	while ( (*c != 0) && i < len) {
		*p = *c;
		c++;
		p++;
		i++;
	}
	*p = 0;
	return 0;
}


